#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : counts.py
# Contents: find pattern spectrum/support border in independent data
# Author  : Christian Borgelt
# History : 2014.12.18 file created from detect/counts.py
#           2015.01.07 added code to create subdirectories
#-----------------------------------------------------------------------
from sys     import argv, path
from os.path import join
from os      import makedirs
from time    import strftime
from random  import seed
UTILDIR  = join('..', 'util')
if UTILDIR  not in path: path.insert(0, UTILDIR)
from id2par  import run
NEURODIR = join('..', 'neuro')
if NEURODIR not in path: path.insert(0, NEURODIR)
from psts    import genpsts
try:
    from ccnovl import coconad
except ImportError:
    COCODIR = join('..', 'coconad')
    if COCODIR not in path: path.insert(0, COCODIR)
    print('Attention: Could not import from PyCoCo library!')
    print('           Using Python replacement (very slow)!')

#-----------------------------------------------------------------------

def counts (ncnt=100, period=3, rate=20, width=0.003,
            target='c', runs=1000, verbose=100, srng=1):
    if srng == 0: srng = 1      # init. seed and get data file name
    name = 'n%d_p%g_r%g_w%g_%c' % (ncnt,period,rate,width*10000,target)
    border = dict()             # create a support border
    hist   = dict()             # and a pattern count histogram
    for i in range(runs):       # repeat the experiments multiple times
        if verbose > 0 and i % verbose == 0:
            print(strftime('%Y.%m.%d %H:%M:%S counts ')+name+' '+str(i))
        seed(srng)              # seed random number generator
        psts = list(zip(range(1,ncnt+1), genpsts(ncnt, period, rate)))
        pats = coconad(psts, target=target, width=width, supp=1.0,
                       zmin=2, zmax=-1, report='a', algo='i', mode='')
        for p,c in pats:        # generate and mine spike trains
            z = len(p)          # update the support border
            if z not in border or c > border[z]: border[z] = c
            s = (z,int(c*5))    # compute histogram bin
            if s in hist: hist[s] += 1
            else:         hist[s]  = 1
        srng += 2               # update the pattern histogram
    if verbose > 0:             # print termination log message
        print(strftime('%Y.%m.%d %H:%M:%S counts ')
                      +name+' '+str(runs)+' done.')
    try: makedirs(join('data','pc'))
    except OSError: pass        # ensure that directory exists
    with open(join('data','pc','bd_'+name+'.txt'), 'w') as out:
        for x in sorted(border.items()):
            out.write('%d %g\n' % x)
    n = 1.0/runs if runs > 0 else 1.0
    with open(join('data','pc','pc_'+name+'.txt'), 'w') as out:
        for x in sorted([(s[0],0.2*s[1],hist[s]*n) for s in hist]):
            out.write('%d %.1f %g\n' % x)
    return border               # write and return the support border

#-----------------------------------------------------------------------

if __name__ == '__main__':
    grids = [ [ ( 'ncnt',   [   100 ] ),
                ( 'period', [     3 ] ),
                ( 'rate',   [    20 ] ),
                ( 'target', [   'c' ] ),
                # 0.0050, 0.0040, 0.0030, 0.0020    1.0 x
                # 0.0055, 0.0044, 0.0033, 0.0022    1.1 x
                # 0.0060, 0.0048, 0.0036, 0.0024    1.2 x
                # 0.0065, 0.0052, 0.0039, 0.0026    1.3 x
                # 0.0070, 0.0056, 0.0042, 0.0028    1.4 x
                # 0.0075, 0.0060, 0.0045, 0.0030    1.5 x
                # 0.0080, 0.0064, 0.0048, 0.0032    1.6 x
                # 0.0085, 0.0068, 0.0051, 0.0034    1.7 x
                # 0.0090, 0.0072, 0.0054, 0.0036    1.8 x
                # 0.0095, 0.0076, 0.0057, 0.0038    1.9 x
                # 0.0100, 0.0080, 0.0060, 0.0040    2.0 x
                ( 'width',  [ 0.0100, 0.0095, 0.0090, 0.0085,
                              0.0080, 0.0076, 0.0075, 0.0072,
                              0.0070, 0.0068, 0.0065, 0.0064,
                              0.0060, 0.0057, 0.0056, 0.0055,
                              0.0054, 0.0052, 0.0051, 0.0050,
                              0.0048, 0.0045, 0.0044, 0.0042,
                              0.0040, 0.0039, 0.0038, 0.0036,
                              0.0034, 0.0033, 0.0032, 0.0030,
                              0.0028, 0.0026, 0.0024, 0.0022,
                              0.0020 ] ),
                ( 'runs',   [ 10000 ] ) ] ]

    run(counts, grids, int(argv[1])-1 if len(argv) > 1 else -1)
